#!/usr/bin/env python3
# -*- coding: utf-8 -*-

### BEGIN LICENSE
# Copyright (C) 2011 ~ 2021 Tianjin KYLIN Information Technology Co., Ltd.
#
# Author:     lixiang <lixiang@kylinos.cn>
# Maintainer: Kylin KMRE Team
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import os
import sys
import json
from ctypes import *

CONF_PATH = "/usr/share/kmre/kmre.conf"
LIB_PATH = "/usr/lib/libkmre.so"

lib_kmre = cdll.LoadLibrary(LIB_PATH)
lib_kmre.get_installed_applist.restype = c_char_p#设置返回值类型为char* 否则默认为int

def has_kmre_env():
    has_kmre = False
    kmre_env = os.getenv("ANDROID_ENV")
    if kmre_env is not None:
        try:
            if (os.path.isfile(CONF_PATH) and os.path.exists(LIB_PATH)):
                res = lib_kmre.is_android_env_installed()
                if (res == True):
                    has_kmre = True
                else:
                    has_kmre = False
        except:
            has_kmre = False
    return has_kmre

def help():
    useage = "$ " + sys.argv[0] + " -u pkgname  #uninstall android apk\n" \
             "$ " + sys.argv[0] + " -l          #list installed android apks"
    print(useage)

# get installed android app list
def get_installed_applist():
    result = lib_kmre.get_installed_applist()
    resultstr = result.decode("utf-8")
    json_obj = json.loads(resultstr)
    #for obj in json_obj:
        #print(obj['app_name'])
        #print(obj['package_name'])
        #print(obj['app_info'])
    return(json_obj)

# uninstall android app
def uninstall_app(pkgname):
    is_installed = False
    for apk_info in get_installed_applist():
        if apk_info['package_name'] == pkgname:
            is_installed = True
    if is_installed == False:
        print('%s is not install' % pkgname)
        return 0

    p = c_char_p(bytes(pkgname, 'utf-8'))
    ret = lib_kmre.uninstall_app(p)
    if ret == 1:
        desktop_path = '%s/.local/share/applications/%s.desktop' % (os.path.expanduser('~'), pkgname)
        png_icon_path = '%s/.local/share/icons/%s.png' % (os.path.expanduser('~'), pkgname)
        svg_icon_path = '%s/.local/share/icons/%s.png' % (os.path.expanduser('~'), pkgname)
        if os.path.exists(desktop_path):
            cmd = 'rm -f %s' % (desktop_path)
            os.system(cmd)
        if os.path.exists(png_icon_path):
            cmd = 'rm -f %s' % (png_icon_path)
            os.system(cmd)
        if os.path.exists(svg_icon_path):
            cmd = 'rm -f %s' % (svg_icon_path)
            os.system(cmd)

    return ret


def main():
    help()
    if len(sys.argv) == 1:
        sys.exit(0)

    kmre_env = has_kmre_env()
    if kmre_env == False:
        print("kmre env is not exists!")
        sys.exit(0)

    if sys.argv[1] == "-l":
        apks_infos = ""
        counts = 0
        json_obj = get_installed_applist()
        for obj in json_obj:
            counts += 1
            apks_infos = '%s AppName: %s \n PkgName: %s \n Version: %s\n---------------------\n' % (apks_infos, obj['app_name'], obj['package_name'], obj['version_name'])
        print("Installed %d apks." % (counts))
        if counts > 0:
            print("Installed apks list:\n", apks_infos)
        return

    if sys.argv[1] == "-u":
        ret = uninstall_app(sys.argv[2])
        print("Uninstall '%s', code result is: %d (%s)" % (sys.argv[2], ret, "Success" if ret == 1 else "Fail"))
        return


if __name__ == "__main__":
    main()
